/**
  *  \file client/widgets/simalliancegrid.cc
  *
  *  FIXME: this widget is very closely related to WTeamGrid.
  */

#include "client/widgets/simalliancegrid.h"
#include "gfx/palette.h"
#include "game/struct.h"
#include "ui/uiglobal.h"
#include "ui/util.h"

WSimAllianceGrid::WSimAllianceGrid()
    : cursor_tick(),
      m_values(),
      m_color(0),
      m_x(2),
      m_y(1)
{
    observe(sig_pos_changed);
    observe(cursor_tick.sig_tick);
    cursor_tick.startPeriodic(250);
    add(cursor_tick, GfxRect());
}

WSimAllianceGrid::~WSimAllianceGrid()
{ }

void
WSimAllianceGrid::getLayoutInfo(LayoutInfo& info)
{
    int size = NUM_OWNERS * font_sizes[FONT_NORMAL].y + 1;
    info.pref_size = info.min_size = GfxPoint(size, size);
    info.flags = 0;
}

void
WSimAllianceGrid::drawContent(GfxCanvas& can)
{
    // Simple version that first draws the whole board, then the cursor.
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());

    const GfxRect& e = getExtent();
    int cell = font_sizes[FONT_NORMAL].y;

    drawBackground(ctx, e);
    for (int pl = 1; pl <= NUM_OWNERS; ++pl) {
        const int y = e.y + cell*(pl-1);
        for (int x = 1; x <= NUM_OWNERS; ++x) {
            const int dx = cell*(x-1) + 1;
            if (x != pl) {
                const uint32_t color = m_values.get(pl, x) ? standard_colors[m_color] : standard_colors[COLOR_GRAYSCALE + 9];
                drawFrameUp   (can, GfxRect(e.x + dx,      y+1, cell-1,        cell-1));
                drawSolidBar  (can, GfxRect(e.x + dx + 1,  y+2, cell-3,        cell-3), color);
            }
        }
    }

    // Cursor
    if (hasState(st_Focused)) {
        if ((cursor_tick.getCounter() & 1) == 0) {
            ctx.setColor(UIColor::tc_Static);
            drawRectangle(ctx, GfxRect(e.x + cell*(m_x-1), e.y + cell*(m_y-1), cell+1, cell+1));
        }
    }

    can.update(e);
}

bool
WSimAllianceGrid::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key && hasState(st_Focused)) {
        switch (e.key.code) {
         case SDLK_LEFT:
            setSelectedWidget();
            moveCursor(0, -1);
            return true;
         case SDLK_RIGHT:
            setSelectedWidget();
            moveCursor(0, +1);
            return true;
         case SDLK_UP:
            setSelectedWidget();
            moveCursor(-1, 0);
            return true;
         case SDLK_DOWN:
            setSelectedWidget();
            moveCursor(+1, 0);
            return true;
         case ' ':
            setSelectedWidget();
            sig_click.raise(m_y, m_x);
            return true;
        }
    }
    if (e.type == ev_Mouse && e.mouse.b != 0 && isInside(GfxPoint(e.mouse.x, e.mouse.y))) {
        setFocusedWidget();
        setSelectedWidget();
        int cell = font_sizes[FONT_NORMAL].y;
        int p = (e.mouse.y - getExtent().y) / cell + 1;
        int t = (e.mouse.x - getExtent().x) / cell + 1;
        if (p > 0 && p <= NUM_OWNERS && t > 0 && t <= NUM_OWNERS) {
            setCursor(p, t);
            if (e.mouse.b & mb_Double) {
                sig_click.raise(m_y, m_x);
            }
        }
        return true;
    }
    return UIWidget::handleEvent(e, second_pass);
}

void
WSimAllianceGrid::onStateChange(int state, bool enable)
{
    UIWidget::onStateChange(state, enable);
    if ((state & st_Focused) != 0) {
        drawWidget();
    }
}

void
WSimAllianceGrid::setCursor(int player, int ally)
{
    player = std::max(1, std::min(int(NUM_OWNERS), player));
    ally   = std::max(1, std::min(int(NUM_OWNERS), ally));
    if (m_x != ally || m_y != player) {
        m_y = player;
        m_x = ally;
        cursor_tick.startPeriodic(250);
        sig_pos_changed.raise();
    }
}

void
WSimAllianceGrid::moveCursor(int dp, int dt)
{
    int x = m_x, y = m_y;
    while (1) {
        x += dt;
        y += dp;
        if (x < 1 || y < 1 || x > NUM_OWNERS || y > NUM_OWNERS) {
            break;
        }
        if (x != y) {
            setCursor(y, x);
            break;
        }
    }
}

void
WSimAllianceGrid::setContent(const GPlayerBitMatrix& mtx, int color)
{
    m_values = mtx;
    m_color = color;
    drawWidget();
}
