/**
  *  \file int/export/expdbf.h
  *  \brief Export to DBF file
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_EXPORT_EXPDBF_H
#define PCC_V2_INT_EXPORT_EXPDBF_H

#include <vector>
#include "int/export.h"

class Stream;

/** Export to DBF (dBASE database) file. */
class IntDbfExporter : public IntExporter {
    Stream& s;                        ///< Stream to write to.

    std::vector<std::size_t> widths;  ///< Field widths.

    uint32_t startPos;                ///< Position of first data record in file.
    uint32_t numRecords;              ///< Number of data records in file.
    uint32_t recordSize;              ///< Size of data record.

    std::vector<char> record;         ///< Buffer for one data record. Size is recordSize.
    uint32_t recordPos;               ///< Next byte to write in data record.
    uint32_t fieldNum;                ///< Next field to write in data record.

    void writeFileHeader();
 public:
    IntDbfExporter(Stream& s);
    void startTable(const IntExportFieldList& fields, const TIntTypeHint types[]);
    void startRecord();
    void addField(IntValue* value, const string_t& name, TIntTypeHint type);
    void endRecord();
    void endTable();
};

#endif
